import cv2
import numpy as np
import glob
import math
import os
import argparse



def three_step_phase_shift(img_input, cxy = None):
    ### img_input:  dictionary of the three images {}"1", "2", "3"}
    ### return the wrapped phase map
    Y = 0
    X = 0
    for i in range(1,4):
        II = img_input[str(i)]
        sigma = 2*(i-1)*np.pi/3
        Y = Y = Y + np.sin(sigma) *II
        X = X + np.cos(sigma) * II
    phase_map = np.arctan2(-Y,X)
    return phase_map

def unwrap_phase(phase_map):
    """
    Unwrap a 2D wrapped phase map.

    Parameters:
    - phase_map: 2D numpy array containing the wrapped phase values.

    Returns:
    - 2D numpy array containing the unwrapped phase values.
    """
    # Unwrap along the first axis (e.g., rows)
    unwrapped_phase_along_first_axis = np.unwrap(phase_map, axis=0)
    
    # Then, unwrap along the second axis (e.g., columns) on the already row-unwrapped phase map
    unwrapped_phase = np.unwrap(unwrapped_phase_along_first_axis, axis=1)
    
    return unwrapped_phase




def unwrap_ref_phase(phase_map, unwrapped_ref):
    phi_u = phase_map + 2*np.pi*np.round((unwrapped_ref-phase_map)/(2*np.pi))
    return phi_u

def manual_fft_filter(phi, filter_freq, x_step, y_step):
    FT=np.fft.fftshift(np.fft.fft2(phi))
    FT[filter_freq[0]:filter_freq[0]+x_step,filter_freq[1]:filter_freq[1]+y_step]=0
    uphi = np.abs(np.fft.ifft2(np.fft.ifftshift(FT)))
    return uphi


import cv2
import numpy as np
import glob
import math
import os
import argparse
import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib.ticker import LinearLocator
from utils_camera import three_step_phase_shift, unwrap_phase, manual_fft_filter, unwrap_ref_phase
from utils_phase_generate import fringe_generation
from scipy import signal
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable



def height_reconstruction(ref_img, target_img):
    ref_wrapped = three_step_phase_shift(ref_img)
    target_wrapped = three_step_phase_shift(target_img)
    ref_unwrapped = unwrap_phase(ref_wrapped)
    target_unwrapped = unwrap_ref_phase(target_wrapped, ref_unwrapped)
    #target_unwrapped = unwrap_phase(target_wrapped)
    # height = np.abs(manual_fft_filter(target_unwrapped)-manual_fft_filter(ref_unwrapped))
    height = np.abs(manual_fft_filter(target_unwrapped - ref_unwrapped))





import cv2
import numpy as np 

def fringe_generation(w,h,f,out_dir):
    """
    Create fringe pattern for three phases shifted by 0, 2π/3, 4π/3
    Parameters:
    - w: image width
    - h: image height
    - f: pitch frequency
    - out_dir: output directory
    """
    width, height = w, h  # Image dimensions
    fringe_frequency = f  # Fringe frequency
    # Create a meshgrid
    x = np.linspace(0, 2 * np.pi * fringe_frequency, width)
    y = np.linspace(0, height, height)
    X, Y = np.meshgrid(x, y)
    fringes = {}
    for i in range(3):
        phase_shift = i * 2 * np.pi / 3  # Phase shift: 0, 2π/3, 4π/3
        pattern = 0.5 + 0.5 * np.cos(X + phase_shift)  # Fringe pattern
        pattern *= 255 / pattern.max()
        fringes[str(i)] = pattern
        cv2.imwrite(out_dir + "sin" + str(i) + ".jpg", pattern)
    return fringes
    